# -*- coding: utf-8 -*-

ABBREVIATIONS = ["dr", "jr", "mr", "mrs", "ms", "msgr", "prof", "sr", "st"]

SUB_PAIRS = [("Esq.", "Esquire")]

ALL_PUNC = u"?!？！.,¡()[]¿…‥،;:—。，、：\n"

TONE_MARKS = u"?!？！"

PERIOD_COMMA = ".,"

COLON = u":"
